#import "iHUBViewController.h"
#import "NSDataCrypto.h"

@implementation iHUBViewController

enum SECTIONS{
    SEC_PORTS=0,
    SEC_CONFIG,
    SEC_CHARGING,
    SEC_LAST
};


static NSString *section_names[]={
    @"iHub Ports",
    @"Port Config",
    @"Charging Current",
};

static int charge_current[]={
    0,1000,1500,2000,2100,2400
};

static int selectedCurrent=0;
static iHUBPortInfo *selectedPort=nil;

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    // Number of sections is the number of region dictionaries
    return SEC_LAST;
}

- (NSString *)tableView:(UITableView *)aTableView titleForHeaderInSection:(NSInteger)section {
    return section_names[section];
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    // Number of rows is the number of names in the region dictionary for the specified section
    size_t nRows=0;
    switch (section) {
        case SEC_PORTS:
            nRows=ports.count;
            break;
        case SEC_CONFIG:
            nRows=configs.count;
            break;
        case SEC_CHARGING:
            nRows=sizeof(charge_current)/sizeof(int);
            break;

        default:
            break;
    }
	return nRows;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    NSError *error=nil;

    switch (indexPath.section) {
        case SEC_PORTS:
        {
            selectedPort=ports[indexPath.row];

//            iHUBRS232Device *port=(iHUBRS232Device *)[dtdev iHUBGetDeviceFormPort:(int)indexPath.row error:nil];
//
//            [port setRS232ConfigWithBaud:115200 dataBits:8 stopBits:IHUB_RS232_STOPBITS_1 parity:IHUB_RS232_PARITY_NONE flowControl:IHUB_RS232_FLOW_NONE error:nil];
//            int written = [port writeData:[@"1111111" dataUsingEncoding:NSASCIIStringEncoding] timeout:10 error:nil];

            break;
        }
        case SEC_CONFIG:
            if(selectedPort==nil)
            {
                ERRMSG(NSLocalizedString(@"No port selected!",nil));
                break;
            }

            if(![dtdev iHUBSetPortConfig:configs[indexPath.row] forPort:selectedPort.portIndex error:&error])
            {
                ERRMSG(NSLocalizedString(@"Command failed",nil));
                break;
            }
            selectedPort.portConfig=configs[indexPath.row];
            [dtdev iHUBSetUSBCharge:[selectedPort.portConfig isEqualToString:kPortConfigUSBCharge] forPort:selectedPort.portIndex error:nil];
            break;
        case SEC_CHARGING:
            if(![dtdev iHUBSetAppleChargingCurrent:charge_current[indexPath.row] error:&error])
            {
                ERRMSG(NSLocalizedString(@"Command failed",nil));
                break;
            }
            selectedCurrent=charge_current[indexPath.row];
            break;

        default:
            break;
    }

    [settingsTable reloadData];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
	UITableViewCell *cell=[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:@"SettingsCell"];
    cell.accessoryType=UITableViewCellAccessoryNone;

    switch (indexPath.section) {
        case SEC_PORTS:
        {
            iHUBPortInfo *port=ports[indexPath.row];
            if(selectedPort==port)
                cell.accessoryType=UITableViewCellAccessoryCheckmark;
            cell.textLabel.text=[NSString stringWithFormat:@"Port %d: %@",port.portIndex,port.portType==IHUB_PORT_TYPE_RS232?@"RS232":@"USB"];
            break;
        }
        case SEC_CONFIG:
            if(selectedPort!=nil && [selectedPort.portConfig isEqualToString:configs[indexPath.row]])
                cell.accessoryType=UITableViewCellAccessoryCheckmark;
            if(indexPath.row==0)
                cell.textLabel.text=@"None";
            else
                cell.textLabel.text=configs[indexPath.row];
            break;
        case SEC_CHARGING:
            if(selectedCurrent==charge_current[indexPath.row])
                cell.accessoryType=UITableViewCellAccessoryCheckmark;
            cell.textLabel.text=[NSString stringWithFormat:@"%dmA",charge_current[indexPath.row]];
            break;

        default:
            break;
    }

	return cell;
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];

    selectedPort=nil;
    selectedCurrent=0;
    
    ports=[dtdev iHUBGetPortsInfo:nil];

    [dtdev iHUBGetAppleChargingCurrent:&selectedCurrent error:nil];

    [settingsTable reloadData];
}


- (void)viewDidLoad
{
    [super viewDidLoad];

	dtdev=[DTDevices sharedDevice];
	[dtdev addDelegate:self];
    
    configs=
    @[
        kPortConfigNone,
        kPortConfigUSBCharge,
        kPortConfigLineaOldUSBSER,
        kPortConfigLineaUSBSER,
        kPortConfigLineaUSB,
        kPortConfigPinpadUSB,
        kPortConfigPinpadUSBSER,
        kPortConfigPrinterESCPOSUSB,
        kPortConfigPrinterESCPOSSerial,
        kPortConfigPrinterFiscalUSB,
        kPortConfigPrinterFiscalOldUSB,
        kPortConfigBarcodeHID,
        kPortConfigBarcodeUSB,
        kPortConfigBarcodeSerial,
        kPortConfigMagtekMSR,
        kPortConfigPrinterGenericESCPOSUSB,
        kPortConfigPrinterGenericESCPOSSerial,
        kPortConfigKeyboardHID
        ];
}

@end
